==> dynatrace_flutter_plugin-3.243.0.tar.gz: lib/src/agent/action/action.dart <==
import '../../agent/action/web_request_timing.dart';
import '../../agent/interface/web_request_timing.dart';
import '../util/logger.dart';
import 'null_actions/null_action_impl.dart';
import '../util/string_utils.dart';
import '../model/platform.dart';
import 'interface/action.dart';
import 'counter.dart';
import 'interface/root_action.dart';

import 'package:flutter/services.dart';

/// Dynatrace root action which has a start- and end-time and can
/// report several values onto it. When finished call leaveAction.
/// This action is able to contain child actions as well.
class DynatraceRootActionImpl extends DynatraceActionImpl
    implements DynatraceRootAction {
  /// Private constructor for creating a root action
  DynatraceRootActionImpl.private(int _key, MethodChannel _methodChannel)
      : super.private(_key, _methodChannel);

  @override
  DynatraceAction enterAction(String? actionName, {Platform? platform}) {
    if (_closed) {
      Logger().d("Action was closed already! Child action will have no effect.",
          logType: LogType.Warning);
      return DynatraceNullAction();
    }

    if (StringUtils.isStringNullOrEmpty(actionName)) {
      Logger().d(
          "Action name was empty or null! Child action will have no effect.",
          logType: LogType.Warning);
      return DynatraceNullAction();
    }

    int newKey = ActionCounter().getCount();

    DynatraceAction action =
        DynatraceActionImpl.private(newKey, _methodChannel);
    ActionCounter().setOpenAction(newKey, action);

    _methodChannel.invokeMethod("enterAction", <String, dynamic>{
      "name": actionName,
      "key": newKey,
      "parent": _key,
      "platform": platform?.index
    });

    return action;
  }

  @override
  Future<WebRequestTiming> createWebRequestTiming(String url) async {
    String? tag = await getRequestTag(url);
    return DynatraceWebRequestTiming(tag, url, _methodChannel);
  }
}

/// Dynatrace action which has a start- and end-time and can
/// report several values onto it. When finished call leaveAction.
class DynatraceActionImpl implements DynatraceAction {
  final int _key;
  final MethodChannel _methodChannel;
  bool _closed = false;

  /// Private constructor for creating a root action
  DynatraceActionImpl.private(this._key, this._methodChannel);

  @override
  void reportError(String? errorName, int? errorCode, {Platform? platform}) {
    if (_closed) {
      Logger().d(
          "Action was closed already! Error can not be reported on this action!",
          logType: LogType.Error);
      return;
    }

    if (!StringUtils.isStringNullOrEmpty(errorName) && errorCode != null) {
      _methodChannel.invokeMethod("reportErrorInAction", <String, dynamic>{
        "key": _key,
        "errorName": errorName,
        "errorCode": errorCode,
        "platform": platform?.index
      });
    } else {
      Logger().d(
          "Error was not reported! ErrorName:$errorName errorCode:$errorCode",
          logType: LogType.Error);
    }
  }

  @override
  void reportEvent(String? eventName, {Platform? platform}) {
    if (_closed) {
      Logger().d(
          "Action was closed already! Event can not be reported on this action!",
          logType: LogType.Error);
      return;
    }
    if (!StringUtils.isStringNullOrEmpty(eventName)) {
      _methodChannel.invokeMethod("reportEventInAction", <String, dynamic>{
        "key": _key,
        "name": eventName,
        "platform": platform?.index
      });
    } else {
      Logger().d(
          "Event was not reported because of missing name! EventName:$eventName",
          logType: LogType.Error);
    }
  }

  @override
  void reportStringValue(String? valueName, String? value,
      {Platform? platform}) {
    if (_closed) {
      Logger().d(
          "Action was closed already! String can not be reported on this action!",
          logType: LogType.Error);
      return;
    }

    if (!StringUtils.isStringNullOrEmpty(valueName)) {
      _methodChannel.invokeMethod(
          "reportStringValueInAction", <String, dynamic>{
        "key": _key,
        "name": valueName,
        "value": value,
        "platform": platform?.index
      });
    } else {
      Logger().d(
          "String was not reported because of missing values! ValueName:$valueName Value:$value",
          logType: LogType.Error);
    }
  }

  @override
  void reportIntValue(String? valueName, int? value, {Platform? platform}) {
    if (_closed) {
      Logger().d(
          "Action was closed already! Integer can not be reported on this action!",
          logType: LogType.Error);
      return;
    }

    if (!StringUtils.isStringNullOrEmpty(valueName) && value != null) {
      _methodChannel.invokeMethod("reportIntValueInAction", <String, dynamic>{
        "key": _key,
        "name": valueName,
        "value": value,
        "platform": platform?.index
      });
    } else {
      Logger().d(
          "Integer was not reported because of missing values! ValueName:$valueName Value:$value",
          logType: LogType.Error);
    }
  }

  @override
  void reportDoubleValue(String? valueName, double? value,
      {Platform? platform}) {
    if (_closed) {
      Logger().d(
          "Action was closed already! Double can not be reported on this action!",
          logType: LogType.Error);
      return;
    }

    if (!StringUtils.isStringNullOrEmpty(valueName) && value != null) {
      _methodChannel.invokeMethod(
          "reportDoubleValueInAction", <String, dynamic>{
        "key": _key,
        "name": valueName,
        "value": value,
        "platform": platform?.index
      });
    } else {
      Logger().d(
          "Double was not reported because of missing values! ValueName:$valueName Value:$value",
          logType: LogType.Error);
    }
  }

  @override
  void leaveAction() {
    if (_closed) {
      Logger().d("Action was closed already! Can not leave the action again!",
          logType: LogType.Error);
      return;
    }

    _closed = true;
    _methodChannel.invokeMethod("leaveAction", <String, dynamic>{"key": _key});
    ActionCounter().removeClosedAction(_key);
  }

  @override
  void cancel() {
    if (_closed) {
      Logger().d(
          "Action was closed already! Can not cancel the action anymore!",
          logType: LogType.Error);
      return;
    }

    _closed = true;
    _methodChannel.invokeMethod("cancelAction", <String, dynamic>{"key": _key});
    ActionCounter().removeClosedAction(_key);
  }

  @override
  Future<String> getRequestTag(String url) async {
    String? tag = await _methodChannel.invokeMethod(
        "getRequestTag", <String, dynamic>{"key": _key, "url": url});

    return tag ?? "";
  }

  @override
  String getRequestTagHeader() {
    return "x-dynatrace";
  }

  @override
  Future<WebRequestTiming> createWebRequestTiming(String url) async {
    String tag = await getRequestTag(url);
    return DynatraceWebRequestTiming(tag, url, _methodChannel);
  }
}

==> dynatrace_flutter_plugin-3.279.2.tar.gz: lib/src/agent/action/action.dart <==
import '../../agent/action/web_request_timing.dart';
import '../../agent/interface/web_request_timing.dart';
import '../util/logger.dart';
import 'null_actions/null_action_impl.dart';
import '../util/string_utils.dart';
import '../model/platform.dart';
import 'interface/action.dart';
import 'counter.dart';
import 'interface/root_action.dart';

import 'package:flutter/services.dart';

/// Dynatrace root action which has a start- and end-time and can
/// report several values onto it. When finished call leaveAction.
/// This action is able to contain child actions as well.
class DynatraceRootActionImpl extends DynatraceActionImpl
    implements DynatraceRootAction {
  /// Private constructor for creating a root action
  DynatraceRootActionImpl.private(int _key, MethodChannel _methodChannel)
      : super.private(_key, _methodChannel);

  @override
  DynatraceAction enterAction(String? actionName, {Platform? platform}) {
    if (_closed) {
      Logger().d(
          "DynatraceRootAction enterAction($actionName): Action was already closed! Child action will have no effect.",
          logType: LogType.Warning);
      return DynatraceNullAction();
    }

    if (StringUtils.isStringNullOrEmpty(actionName)) {
      Logger().d(
          "DynatraceRootAction enterAction($actionName): Action name was empty or null! Child action will have no effect.",
          logType: LogType.Warning);
      return DynatraceNullAction();
    }

    Logger().d("DynatraceRootAction enterAction($actionName)",
        logType: LogType.Info);
    int newKey = ActionCounter().getCount();

    DynatraceAction action =
        DynatraceActionImpl.private(newKey, _methodChannel);
    ActionCounter().setOpenAction(newKey, action);

    _methodChannel.invokeMethod("enterAction", <String, dynamic>{
      "name": actionName,
      "key": newKey,
      "parent": _key,
      "platform": platform?.index
    });

    return action;
  }

  @override
  Future<WebRequestTiming> createWebRequestTiming(String url) async {
    Logger().d("DynatraceRootAction createWebRequestTiming($url)",
        logType: LogType.Info);
    String? tag = await getRequestTag(url);
    return DynatraceWebRequestTiming(tag, url, _methodChannel);
  }
}

/// Dynatrace action which has a start- and end-time and can
/// report several values onto it. When finished call leaveAction.
class DynatraceActionImpl implements DynatraceAction {
  final int _key;
  final MethodChannel _methodChannel;
  bool _closed = false;

  /// Private constructor for creating a root action
  DynatraceActionImpl.private(this._key, this._methodChannel);

  @override
  void reportError(String? errorName, int? errorCode, {Platform? platform}) {
    if (_closed) {
      Logger().d(
          "DynatraceAction reportError($errorName, $errorCode): Action was already closed! Error cannot be reported on this action!",
          logType: LogType.Error);
      return;
    }

    if (!StringUtils.isStringNullOrEmpty(errorName) && errorCode != null) {
      Logger().d("DynatraceAction reportError($errorName $errorCode)",
          logType: LogType.Info);
      _methodChannel.invokeMethod("reportErrorInAction", <String, dynamic>{
        "key": _key,
        "errorName": errorName,
        "errorCode": errorCode,
        "platform": platform?.index
      });
    } else {
      Logger().d(
          "DynatraceAction reportError(errorName, errorCode): Error was not reported because errorName is null or empty and or error code is null! \nerrorName: $errorName \nerrorCode: $errorCode",
          logType: LogType.Error);
    }
  }

  @override
  void reportEvent(String? eventName, {Platform? platform}) {
    if (_closed) {
      Logger().d(
          "DynatraceAction reportEvent($eventName): Action was already closed! Event cannot be reported on this action!",
          logType: LogType.Error);
      return;
    }
    if (!StringUtils.isStringNullOrEmpty(eventName)) {
      Logger()
          .d("DynatraceAction reportEvent($eventName)", logType: LogType.Info);
      _methodChannel.invokeMethod("reportEventInAction", <String, dynamic>{
        "key": _key,
        "name": eventName,
        "platform": platform?.index
      });
    } else {
      Logger().d(
          "DynatraceAction reportEvent(eventName): Event not reported because eventName is null or empty! \neventName: $eventName",
          logType: LogType.Error);
    }
  }

  @override
  void reportStringValue(String? valueName, String? value,
      {Platform? platform}) {
    if (_closed) {
      Logger().d(
          "DynatraceAction reportStringValue($valueName, $value): Action was already closed! String cannot be reported on this action!",
          logType: LogType.Error);
      return;
    }

    if (!StringUtils.isStringNullOrEmpty(valueName)) {
      Logger().d("DynatraceAction reportStringValue($valueName, $value)",
          logType: LogType.Info);
      _methodChannel.invokeMethod(
          "reportStringValueInAction", <String, dynamic>{
        "key": _key,
        "name": valueName,
        "value": value,
        "platform": platform?.index
      });
    } else {
      Logger().d(
          "DynatraceAction reportStringValue(valueName, value): String not reported because valueName is null or empty! \nvalueName: $valueName \nvalue: $value",
          logType: LogType.Error);
    }
  }

  @override
  void reportIntValue(String? valueName, int? value, {Platform? platform}) {
    if (_closed) {
      Logger().d(
          "DynatraceAction reportIntValue($valueName, $value): Action was already closed! Integer cannot be reported on this action!",
          logType: LogType.Error);
      return;
    }

    if (!StringUtils.isStringNullOrEmpty(valueName) && value != null) {
      Logger().d("DynatraceAction reportIntValue($valueName, $value)",
          logType: LogType.Info);
      _methodChannel.invokeMethod("reportIntValueInAction", <String, dynamic>{
        "key": _key,
        "name": valueName,
        "value": value,
        "platform": platform?.index
      });
    } else {
      Logger().d(
          "DynatraceAction reportIntValue(valueName, value): Integer not reported because valueName is null or empty! \nvalueName: $valueName \nvalue: $value",
          logType: LogType.Error);
    }
  }

  @override
  void reportDoubleValue(String? valueName, double? value,
      {Platform? platform}) {
    if (_closed) {
      Logger().d(
          "DynatraceAction reportDoubleValue($valueName, $value): Action was already closed! Double cannot be reported on this action!",
          logType: LogType.Error);
      return;
    }

    if (!StringUtils.isStringNullOrEmpty(valueName) && value != null) {
      Logger().d("DynatraceAction reportDoubleValue($valueName, $value)",
          logType: LogType.Info);
      _methodChannel.invokeMethod(
          "reportDoubleValueInAction", <String, dynamic>{
        "key": _key,
        "name": valueName,
        "value": value,
        "platform": platform?.index
      });
    } else {
      Logger().d(
          "DynatraceAction reportDoubleValue(valueName, value): Double was not reported because valueName is null or empty! \nvalueName: $valueName \nvalue: $value",
          logType: LogType.Error);
    }
  }

  @override
  void leaveAction() {
    if (_closed) {
      Logger().d(
          "DynatraceAction leaveAction(): Action was already closed! Unable to leave the action again!",
          logType: LogType.Error);
      return;
    }

    Logger().d("DynatraceAction leaveAction()", logType: LogType.Info);
    _closed = true;
    _methodChannel.invokeMethod("leaveAction", <String, dynamic>{"key": _key});
    ActionCounter().removeClosedAction(_key);
  }

  @override
  void cancel() {
    if (_closed) {
      Logger().d(
          "DynatraceAction cancel(): Action was already closed! Unable to cancel the action anymore!",
          logType: LogType.Error);
      return;
    }

    Logger().d("DynatraceAction cancel()", logType: LogType.Info);
    _closed = true;
    _methodChannel.invokeMethod("cancelAction", <String, dynamic>{"key": _key});
    ActionCounter().removeClosedAction(_key);
  }

  @override
  Future<String> getRequestTag(String url) async {
    Logger().d("DynatraceAction getRequestTag($url)", logType: LogType.Info);
    String? tag = await _methodChannel.invokeMethod(
        "getRequestTag", <String, dynamic>{"key": _key, "url": url});

    return tag ?? "";
  }

  @override
  String getRequestTagHeader() {
    Logger().d("DynatraceAction getRequestTagHeader()", logType: LogType.Info);
    return "x-dynatrace";
  }

  @override
  Future<WebRequestTiming> createWebRequestTiming(String url) async {
    Logger().d("DynatraceAction createWebRequestTiming($url)",
        logType: LogType.Info);
    String tag = await getRequestTag(url);
    return DynatraceWebRequestTiming(tag, url, _methodChannel);
  }
}
